type register = R of int

type ic = Signed of (unit -> int)
type uc = Unsigned of (unit -> int)
type oc = Relative of (unit -> int)
type lc = Absolute of (unit -> int)

type a_riiu = AR of register | AI of ic | AIU of uc

type b_ri = BR of register | BI of ic

type arith_op = RAdd | RSub | RMul | RDiv | RCmp | RMod
type logical_op = RAnd | ROr | RXor | RBic
type logical_sh = RLsh | RAsh
type mem_op = RLdw | RLdb | RStw | RStb | RPop | RPsh
type test_op = RBeq | RBne | RBlt | RBge | RBgt | RBle

type syscall = 
    Sys_io_rd_chr
  | Sys_io_rd_int
  | Sys_io_wr_chr
  | Sys_io_wr_int
  | Sys_gc_init of register
  | Sys_gc_alloc of register
  | Sys_get_total_mem_size
  | Sys_exit

type instruction = 
    IArith of arith_op * register * register * a_riiu
  | ILog of logical_op * register * register * a_riiu
  | ISh of logical_sh * register * register * b_ri
  | IMem of mem_op * register * register * ic
  | ITest of test_op * register * oc
  | Chk of register * a_riiu
  | Bsr of oc
  | Jsr of lc
  | Ret of register
  | Break
  | Syscall of register * syscall

val string_of_register : register -> string
val string_of_instruction : instruction -> string

val word_size : int
