open Symbol

type ext_type = 
    TInt
  | TArrow of (ext_type list) * ext_type
  | TClass of string
and int_type =
    TExt of ext_type
  | TNull
  | TGamma of env
  | TBad
and env = TEnv of binding list
and binding = TBind of string * symbol * kind * int_type
and kind = KVar | KFun | KClass | KField

let pr_string,gt_string = 
  let buf = Buffer.create 10 in
    Buffer.clear buf;
    let pr_string s =
      Buffer.add_string buf s
    and gt_string () =
      let s = Buffer.contents buf in
	Buffer.clear buf;
	s
    in pr_string,gt_string

let rec print_ext_type = function
    TInt -> pr_string "Int"
  | TArrow(ts,t) -> 
      pr_string "(";
      begin
	  match ts with
	      [] -> ()
	    | [t] -> print_ext_type t
	    | t::ts ->
		print_ext_type t;
		List.iter (fun t -> pr_string ", ";print_ext_type t) ts
      end;
      pr_string ") => ";
      print_ext_type t
  | TClass(id) -> pr_string id
and print_int_type = function
    TExt(t) -> print_ext_type t
  | TNull -> pr_string "Null"
  | TGamma(e) -> pr_string "/";print_env e;pr_string "/"
  | TBad -> pr_string "<BadType>"
and print_env (TEnv(bs)) =
  match bs with
      [] -> ()
    | [b] -> print_binding b
    | b::bs -> 
	print_binding b;
	List.iter (fun b -> pr_string ", ";print_binding b) bs
and print_binding (TBind(id,sym_id,k,t)) =
  pr_string id;pr_string " as ";pr_string (string_of_symbol sym_id);pr_string ":";print_kind k;pr_string "[";print_int_type t;pr_string "]"
and print_kind = function
    KVar -> pr_string "Var"
  | KFun -> pr_string "Fun"
  | KClass -> pr_string "Class"
  | KField -> pr_string "Field"

let string_of_ext_type t = 
  print_ext_type t;
  gt_string ()
and string_of_int_type t =
  print_int_type t;
  gt_string ()
and string_of_env e =
  print_env e;
  gt_string ()
and string_of_binding b =
  print_binding b;
  gt_string ()
and string_of_kind k =
  print_kind k;
  gt_string ()

let print_ext_type t = print_string (string_of_ext_type t)
and print_int_type t = print_string (string_of_int_type t)
and print_env e = print_string (string_of_env e)
and print_binding b = print_string (string_of_binding b)
and print_kind k = print_string (string_of_kind k)
